#!/usr/bin/env python3

"""Extract and copy the content of reStructuredText docstrings of Python scripts to a Grav CMS repository.

Description
-----------

This script extracts and copies the content of reStructuredText docstrings of Python scripts to a Grav CMS repository.

Contrary to the other pipelines, this script does not copy one file to one page in Grav, but creates a tree of pages
below one page (given by the pipeline header). it processes all ``run.py`` and ``__init__.py`` files.

The PIPELINE and PIPELINE_SOURCE options are used in the same way as in ``run_markdown_pipeline.py``.

In addition, PIPELINE_IMAGES specifies a directory where the images from the docstrings are located and PIPELINE_HEADER
and PIPELINE_FOOTER options point to templates which are prepended and appended to each page.

With the PIPELINE_REFS YML file, you can specify replacements for the references in the rst code.

Please refer to https://git.opencarp.org/openCARP/experiments for an example setup.

Usage
-----

.. argparse::
    :module: facile_rs.run_docstring_pipeline
    :func: create_parser
    :prog: run_docstring_pipeline.py

"""
import ast
import logging
import os
import re
import shutil
from pathlib import Path

import frontmatter
import pypandoc
import yaml
from PIL import Image
from resizeimage import resizeimage

from .utils import cli
from .utils.grav import collect_pages

logger = logging.getLogger(__file__)

FIGURE_PATTERN = r'\.\. figure\:\:\s(.+?)\s'

REF_PATTERN = r'\:ref\:\`(.+?)\s\<(.+?)\>\`'

METADATA_PATTERN = r'__(.*)__ = [\']([^\']*)[\']'
METADATA_RUN_PATTERN = r'EXAMPLE_(.*) = [\']([^\']*)[\']'


def create_parser(add_help=True):
    parser = cli.Parser(add_help=add_help)

    parser.add_argument('--grav-path', dest='GRAV_PATH', required=True,
                        help='Path to the grav repository directory.')
    parser.add_argument('--pipeline', dest='PIPELINE', required=True,
                        help='Name of the pipeline as specified in the GRAV metadata.')
    parser.add_argument('--pipeline-source', dest='PIPELINE_SOURCE', required=True,
                        help='Path to the source directory for the pipeline.')
    parser.add_argument('--pipeline-images', dest='PIPELINE_IMAGES',
                        help='Path to the images directory for the pipeline.')
    parser.add_argument('--pipeline-header', dest='PIPELINE_HEADER',
                        help='Path to the header template.')
    parser.add_argument('--pipeline-footer', dest='PIPELINE_FOOTER',
                        help='Path to the footer template.')
    parser.add_argument('--pipeline-refs', dest='PIPELINE_REFS',
                        help='Path to the refs yaml file.')
    parser.add_argument('--output-html', action='store_true', dest='OUTPUT_HTML',
                        help='Output HTML files instead of markdown')
    parser.add_argument('--mathjax-location', dest='MATHJAX_LOCATION',
                        default='https://cdn.jsdelivr.net/npm/mathjax@3/es5/tex-mml-chtml.js',
                        help='Location of the MathJax script for math rendering in HTML output. '
                             'This option is only used if --output-html is set. '
                             'Set to empty string to disable MathJax.')
    parser.add_argument('--log-level', dest='LOG_LEVEL', default='WARN',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='LOG_FILE',
                        help='Path to the log file')
    return parser


def main(args):
    # compile patterns
    ref_pattern = re.compile(REF_PATTERN)
    figure_pattern = re.compile(FIGURE_PATTERN)

    # get the source path
    source_path = Path(args.PIPELINE_SOURCE).expanduser()

    # get the images path
    if args.PIPELINE_IMAGES:
        images_path = Path(args.PIPELINE_IMAGES).expanduser()
    else:
        images_path = None

    # read header
    if args.PIPELINE_HEADER:
        header = Path(args.PIPELINE_HEADER).expanduser().read_text()
    else:
        header = ''
    if args.OUTPUT_HTML:
        header_prefix = "<html><head><meta charset=\"utf-8\">"
        # Add mathjax script for math rendering
        if args.MATHJAX_LOCATION:
            header_prefix += f"<script id=\"MathJax-script\" async src=\"{args.MATHJAX_LOCATION}\"></script>"
        header_prefix += "</head><body>"
        header = header_prefix + header

    # read footer
    if args.PIPELINE_FOOTER:
        footer = Path(args.PIPELINE_FOOTER).expanduser().read_text()
    else:
        footer = ''
    if args.OUTPUT_HTML:
        footer = footer + "</body></html>"

    # read refs
    if args.PIPELINE_REFS:
        refs_path = Path(args.PIPELINE_REFS).expanduser()
        refs = yaml.safe_load(refs_path.read_text())
    else:
        refs = {}

    # loop over all experiments
    for page_path, page, _ in collect_pages(args.GRAV_PATH, args.PIPELINE):
        for root, dirs, files in os.walk(source_path):
            # skip source_path itself
            if root != source_path:
                root_path = Path(root)
                run_path = root_path / 'run.py'
                init_path = root_path / '__init__.py'
                if args.OUTPUT_HTML:
                    md_path = Path(root.replace(str(source_path), str(page_path.parent)).lower()) / 'default.html'
                else:
                    md_path = Path(root.replace(str(source_path), str(page_path.parent)).lower()) / 'default.md'

                if 'run.py' in files:
                    # read the __init__.py file and obtain the metadata
                    with open(init_path) as f:
                        metadata = dict(re.findall(METADATA_PATTERN, f.read()))

                    # read the run.py file and obtain the metadata
                    with open(run_path) as f:
                        metadata_run = dict(re.findall(METADATA_RUN_PATTERN, f.read()))
                    titleString = ''
                    if 'DESCRIPTIVE_NAME' in metadata_run.keys():
                        titleString = titleString + '<h1>' + metadata_run.get('DESCRIPTIVE_NAME') + '</h1>\n'
                    titleString = titleString \
                            + '<i>See <a href="https://git.opencarp.org/openCARP/experiments/-/blob/master/' \
                            + str(run_path) + '" target="_blank">code</a> in GitLab.</i><br/>\n'
                    if 'AUTHOR' in metadata_run.keys():
                        titleString = titleString + '<i>Author: ' + metadata_run.get('AUTHOR') + '</i>\n'

                    # read the run.py file and obtain the docstring
                    with open(run_path) as f:
                        py_string = f.read()
                        module = ast.parse(py_string)
                        docstring = ast.get_docstring(module)

                    # search for :ref:
                    for m in ref_pattern.finditer(docstring):
                        text, ref = m.group(1), m.group(2)

                        if ref in refs and refs[ref] is not None:
                            target = refs[ref]
                            docstring = docstring.replace(m.group(0), f'`{text} <{target}>`_')
                        else:
                            logger.warning(f'Reference {m.group(0)} missing')
                            docstring = docstring.replace(m.group(0), text)

                    # search for .. figure::
                    images = []
                    for m in figure_pattern.finditer(docstring):
                        figure = m.group(1)
                        image = figure.replace('/images/', '')
                        images.append(image)
                        if args.OUTPUT_HTML:
                            docstring = docstring.replace(figure, image)
                        else:
                            docstring = docstring.replace(figure, str(Path(root_path.name.lower()) / image))

                    # append image from the metadata to the image list
                    if metadata.get('image'):
                        image_name = metadata.get('image').replace('/images/', '')
                        thumb_name = 'thumb_' + image_name
                        with open(images_path / image_name, 'r+b') as f:
                            with Image.open(f) as image:
                                w, h = image.size
                                if w > 200:
                                    cover = resizeimage.resize_width(image, 200)
                                cover.save(images_path / thumb_name, image.format)
                        images.append(thumb_name)

                    # create content from the docstring using pandoc
                    # we should probably add '--shift-heading-level-by=1' to extra_args but it doesn't
                    # seem to be supported by our pandoc version
                    body = pypandoc.convert_text(docstring, to='html', format='rst',
                                                 extra_args=['--mathjax', '--wrap=preserve'])

                    # convert RST section headers to level 2 headings
                    body = body.replace('<h1 id=', '<h2 id=')
                    body = body.replace('</h1>', '</h2>')

                    q2a_tags = metadata.get('q2a_tags', '')
                    wrapped_q2a_tags = ''
                    if q2a_tags:
                        wrapped_q2a_tags += f'[q2a tags="{q2a_tags}"]\n'

                    content = header + titleString + body + wrapped_q2a_tags + footer

                    # create directories in the grav tree
                    md_path.parent.mkdir(parents=True, exist_ok=True)

                    # update or create markdown file
                    title = metadata.get('title', '')
                    description = metadata.get('description', '')
                    image = metadata.get('image', '').replace('/images/', '')
                    thumb_name = ''
                    if image:
                        thumb_name = 'thumb_' + image


                    try:
                        page = frontmatter.load(md_path)
                        page.content = content
                        page['title'] = title
                        page['description'] = description
                        page['image'] = thumb_name

                    except FileNotFoundError:
                        page = frontmatter.Post(content, title=title, description=description, image=thumb_name)

                    # write the grav file
                    logger.info('writing to %s', md_path)
                    if args.OUTPUT_HTML:
                        md_path.write_text(content)
                    else:
                        md_path.write_text(frontmatter.dumps(page))

                    # copy images
                    if images_path is not None:
                        for image in images:
                            source = images_path / image
                            destination = md_path.parent / image

                            try:
                                shutil.copy(source, destination)
                                logger.debug(f'Copy image {source} to {destination}')
                            except FileNotFoundError:
                                logger.warning(f'Image {source} missing')

                elif '__init__.py' in files:
                    # create directories in the grav tree
                    md_path.parent.mkdir(parents=True, exist_ok=True)

                    # read the __init__.py file and obtain the metadata
                    with open(init_path) as f:
                        metadata = dict(re.findall(METADATA_PATTERN, f.read()))

                    content = ''
                    metadata = {
                        'title': metadata.get('title', ''),
                        'cards': {
                            'items': '@self.children'
                        }
                    }
                    page = frontmatter.Post(content, **metadata)

                    # write the grav file
                    logger.info('writing to %s', md_path)
                    md_path.write_text(frontmatter.dumps(page))


def main_deprecated():
    cli.main_deprecated(__name__)


if __name__ == "__main__":
    main_deprecated()



#!/usr/bin/env python3

"""Create a CFF citation file from a CodeMeta metadata file

Description
-----------

This script takes a CodeMeta metadata file as input and generates a CFF (Citation File Format) citation file.
The CFF file contains information about the software, including its authors, contributors, and other metadata.

The script accepts command-line arguments to specify the locations of the CodeMeta metadata file, additional
codemeta JSON files for creators and contributors and the path to the output CFF file.

Usage
-----

.. argparse::
   :module: facile_rs.create_cff
   :func: create_parser
   :prog: create_cff.py

Example usage
-------------

.. code-block:: bash

    python3 create_cff.py \\
        --codemeta-location /path/to/codemeta.json \\
        --creators-location /path/to/creators.json \\
        --cff-path /path/to/output.cff

"""
from pathlib import Path

from rich import print

from .utils import cli
from .utils.metadata import CffMetadata, CodemetaMetadata


def create_parser(add_help=True):
    parser = cli.Parser(add_help=add_help)

    parser.add_argument('--codemeta-location', dest='CODEMETA_LOCATION', required=True,
                        help='Locations of the main codemeta.json JSON file')
    parser.add_argument('--creators-locations', '--creators-location', dest='CREATORS_LOCATIONS',
                        action='append', default=[],
                        help='Locations of codemeta JSON files for additional creators')
    parser.add_argument('--contributors-locations', '--contributors-location', dest='CONTRIBUTORS_LOCATIONS',
                        action='append', default=[],
                        help='Locations of codemeta JSON files for additional contributors')
    parser.add_argument('--cff-path', dest='CFF_PATH',
                        help='Path to the cff output file')
    parser.add_argument('--no-sort-authors', dest='SORT_AUTHORS', action='store_false',
                        help='Do not sort authors alphabetically, keep order in codemeta.json file')
    parser.set_defaults(sort_authors=True)
    parser.add_argument('--log-level', dest='LOG_LEVEL', default='WARN',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='LOG_FILE',
                        help='Path to the log file')
    return parser

def main(args):
    codemeta = CodemetaMetadata()
    codemeta.fetch(args.CODEMETA_LOCATION)
    codemeta.fetch_authors(args.CREATORS_LOCATIONS)
    codemeta.fetch_contributors(args.CONTRIBUTORS_LOCATIONS)
    codemeta.remove_doubles()
    if args.SORT_AUTHORS:
        codemeta.sort_persons()

    cff_metadata = CffMetadata(codemeta.data)
    cff_yaml = cff_metadata.to_yaml()

    if args.CFF_PATH:
        Path(args.CFF_PATH).expanduser().write_text(cff_yaml)
    else:
        print(cff_yaml)


def main_deprecated():
    cli.main_deprecated(__name__)


if __name__ == "__main__":
    main_deprecated()



from os import path

import pytest
import yaml

from facile_rs.utils.metadata import CffMetadata, CodemetaMetadata
from facile_rs.utils.metadata.cff import schema_org_identifier_to_cff

# Get current script location
SCRIPT_DIR = path.dirname(path.realpath(__file__))


def create_metadata(codemeta_filename):
    codemeta = CodemetaMetadata()
    codemeta.fetch(path.join(SCRIPT_DIR, codemeta_filename))
    codemeta.compute_names()
    metadata = CffMetadata(codemeta.data)
    return codemeta, metadata


def test_schemaorg_identifier_to_cff():
    schemaorg_id = [{
        "@type": "PropertyValue",
        "propertyID": "DOI",
        "value": "10.35097/1952"
    },
    {
        "@type": "PropertyValue",
        "propertyID": "RADAR",
        "value": "gNzfgsCdFVucufDC"
    }]
    expected_result_0 = {
        "type": "doi",
        "value": "10.35097/1952"
    }
    assert schema_org_identifier_to_cff(schemaorg_id[0]) == expected_result_0
    assert schema_org_identifier_to_cff(schemaorg_id[1]) == {}


@pytest.mark.parametrize("codemeta_filename", [
    'codemeta_test.json',
    'codemeta_isbasedon.json'
])
def test_init(codemeta_filename):
    codemeta, metadata = create_metadata(codemeta_filename)
    assert metadata.data == codemeta.data

@pytest.mark.parametrize(("codemeta_filename", "cff_ref_filename"), [
    ('codemeta_test.json', 'cff_ref.yml'),
    ('codemeta_isbasedon.json', 'cff_isbasedon_ref.yml')
])
def test_conversion(codemeta_filename, cff_ref_filename):
    _, metadata = create_metadata(codemeta_filename)
    print(metadata.to_yaml())
    with open(path.join(SCRIPT_DIR, cff_ref_filename)) as f:
        assert yaml.safe_load(metadata.to_yaml()) == yaml.safe_load(f)



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Python modules --
# system

import pathlib
import sys
from importlib.metadata import version as get_version

# 3rd party
from sphinx_pyproject import SphinxConfig

FACILERS = pathlib.Path(__file__).parents[3]
sys.path.insert(0, FACILERS.resolve().as_posix())

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

myproject_version = get_version("facile-rs")
config = SphinxConfig("../../../pyproject.toml", globalns=globals(), config_overrides = {"version": myproject_version})

project = config.name
author = config.author
description = config.description

# project = 'FACILE-RS'
# copyright = '2024, FACILE-RS'
# author = 'FACILE-RS'

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
  'sphinx.ext.duration',
  'sphinx.ext.doctest',
  'autodoc2',
  'sphinx.ext.autosummary',
  'myst_parser',
  'sphinxarg.ext'
  ]

autodoc2_packages = ["../../../facile_rs",]

templates_path = ['_templates']
exclude_patterns = []

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'classic'
html_theme_options = {
  # 'nosidebar': False,
  'sidebarwidth': '350px',
  'collapsiblesidebar': True,
  }
html_copy_source = False
globaltoc_collapse = True
modindex_common_prefix = [
  'facile_rs.',
  'facile_rs.utils.',
  'facile_rs.utils.metadata.'
  ]
html_sidebars = { '**': ['localtoc.html', 'globaltoc.html', 'relations.html', 'sourcelink.html', 'searchbox.html'] }


# These two options don't seem to have any effect
add_module_names = False
# python_use_unqualified_type_names = True

#html_static_path = ['_static']



import json
import logging

from ..http import fetch_dict

logger = logging.getLogger(__file__)


class CodemetaMetadata:

    """A class for storing and manipulating metadata in the CodeMeta format"""

    def __init__(self):
        """Initialize metadata set as an empty dictionary."""
        self.data = {}

    def fetch(self, location):
        """Update metadata set with data fetched from a CodeMeta file

        :param location: URL or path to the CodeMeta JSON file
        :type location: str
        """
        if location:
            self.data.update(fetch_dict(location))

    def fetch_authors(self, locations):
        """Fetch authors from CodeMeta files and update metadata set

        :param locations: list of URL or paths to CodeMeta files
        :param locations: list
        """
        if locations:
            if 'author' not in self.data:
                self.data['author'] = []
            # Case when there is a unique author not contained in a list
            elif isinstance(self.data['author'], dict):
                self.data['author'] = [self.data['author']]
            for location in locations:
                self.data['author'] += fetch_dict(location).get('author', [])

    def fetch_contributors(self, locations):
        """Fetch contributors from CodeMeta files and update metadata set

        :param locations: list of URL or paths to CodeMeta files
        :param locations: list
        """
        if locations:
            if 'contributor' not in self.data:
                self.data['contributor'] = []
            # Case when there is a unique contributor not contained in a list
            elif isinstance(self.data['contributor'], dict):
                self.data['contributor'] = [self.data['contributor']]
            for location in locations:
                self.data['contributor'] += fetch_dict(location).get('contributor', [])

    def compute_names(self):
        """Add full name of authors and contributors in metadata set from given name and family name,
        under the dictionary key 'name'.
        """
        for key in ['author', 'contributor']:
            if key in self.data:
                for thing in self.data[key]:
                    if 'name' not in thing and ('givenName' in thing and 'familyName' in thing):
                        thing['name'] = '{} {}'.format(thing['givenName'], thing['familyName'])

    def remove_doubles(self):
        """Remove duplicates in authors and contributors lists, comparing names (key: name), concatenation of keys
        givenName and familyName, and ids (key: @id).
        """
        for key in ['author', 'contributor']:
            if key in self.data:
                ids = set()
                names = set()
                givenAndFamilyNames = set()
                things = []
                if not isinstance(self.data[key], list):
                    data_list = [self.data[key]]
                else:
                    data_list = self.data[key]
                for thing in data_list:
                    thing_id = thing.get('@id')
                    thing_name = thing.get('name')
                    thing_givenAndFamilyName = None
                    if 'givenName' in thing and 'familyName' in thing:
                        thing_givenAndFamilyName = '{} {}'.format(thing['givenName'], thing['familyName'])
                    if thing_id in ids or thing_name in names or thing_givenAndFamilyName in givenAndFamilyNames:
                        pass
                    else:
                        things.append(thing)
                        if thing_id is not None:
                            ids.add(thing_id)
                        if thing_name is not None:
                            names.add(thing_name)
                        if thing_givenAndFamilyName is not None:
                            givenAndFamilyNames.add(thing_givenAndFamilyName)
                self.data[key] = things

    def sort_persons(self):
        """Sort authors and contributors alphabetically based on family name."""
        def get_key(item):
            key = item.get('familyName', item.get('name', ''))

            if item.get('@type') == 'Organization':
                # put organzations first unless they have an additionalType
                if item.get('additionalType') is None:
                    key = '!' + key
                else:
                    key = '~' + key

            return key

        if 'author' in self.data:
            self.data['author'] = sorted(self.data['author'], key=get_key)
        if 'contributor' in self.data:
            self.data['contributor'] = sorted(self.data['contributor'], key=get_key)

    def update_identifier(self, new_doi, new_archive_id='', archive_type=None, keep_previous_doi=False):
        """Update the id field and identifier list with a new DOI and archive ID (optional).

        If keep_previous_doi is True, the previous DOI is kept in the Codemeta metadata in the field "isBasedOn".

        :param new_doi: new DOI to add (e.g., '10.5281/zenodo.1234567')
        :type new_doi: str
        :param new_archive_id: new archive ID to add (e.g., RADAR ID or Zenodo ID)
        :type new_archive_id: str
        :param archive_type: type of the archive, e.g., 'RADAR' or 'Zenodo'. If None, no archive ID is added.
        :type archive_type: str or None
        :param keep_previous_doi: whether to keep previous DOI in the Codemeta metadata
        :type keep_previous_doi: bool
        """

        if archive_type not in [None, 'RADAR', 'Zenodo']:
            raise ValueError("archive_type must be either None, 'RADAR', or 'Zenodo'")

        previous_id = self.data.get('@id', '')
        self.data['@id'] = 'https://doi.org/' + new_doi

        doi_entry = {
            '@type': 'PropertyValue',
            'propertyID': 'DOI',
            'value': new_doi,
            'description': 'The DOI for version {version} of this work'.format(**self.data)
        }
        archive_entry = None
        if archive_type and new_archive_id:
            archive_entry = {
                '@type': 'PropertyValue',
                'propertyID': archive_type,
                'value': new_archive_id
            }

        if 'identifier' in self.data:
            # Ensure that identifier field is a list
            if not isinstance(self.data['identifier'], list):
                self.data['identifier'] = [self.data['identifier']]
            previous_doi = previous_id.replace('https://doi.org/', '')
            found_doi = False
            found_archive = False
            if keep_previous_doi:
                # Get or create the isBasedOn field
                isBasedOn = self.data.get('isBasedOn', [])
                if not isinstance(isBasedOn, list):
                    isBasedOn = [isBasedOn]
            for ind, identifier in enumerate(self.data['identifier']):
                if not found_doi and identifier.get('propertyID') == 'DOI' \
                    and identifier.get('value') == previous_doi:
                    if keep_previous_doi:
                        isBasedOn.append(
                            {
                                '@type': 'CreativeWork',
                                'identifier': identifier
                            }
                        )
                    self.data['identifier'][ind] = doi_entry
                    found_doi = True
                elif archive_entry and not found_archive and identifier.get('propertyID') == archive_type:
                    identifier['value'] = new_archive_id
                    found_archive = True
            if not found_doi:
                self.data['identifier'].append(doi_entry)
            if archive_entry and not found_archive:
                self.data['identifier'].append(archive_entry)
            if keep_previous_doi and isBasedOn:
                self.data['isBasedOn'] = isBasedOn
        else:
            self.data['identifier'] = [doi_entry]
            if archive_entry:
                self.data['identifier'].append(archive_entry)


    def to_json(self):
        """Dump metadata set as JSON-formatted string.

        :return: metadata set as JSON-formatted string
        :rtype: str
        """
        return json.dumps(self.data, indent=2, ensure_ascii=False)



#!/usr/bin/env python3

"""Create an empty archive in Zenodo to reserve a DOI and a Zenodo ID.

Description
-----------

This script creates an empty archive in Zenodo in order to reserve a DOI and a Zenodo ID.
Both are stored in the CodeMeta metadata file provided as input and can be later used by the script ``create_zenodo.py``
to populate the Zenodo archive.

Optionally, the script can create a new version from an existing Zenodo record.
See the option --zenodo-version-update for more details.

Usage
-----

.. argparse::
    :module: facile_rs.prepare_zenodo
    :func: create_parser
    :prog: prepare_zenodo.py

"""
from pathlib import Path

from rich import print

from .utils import cli
from .utils.exceptions import ParserError
from .utils.http import is_remote
from .utils.metadata import CodemetaMetadata, ZenodoMetadata
from .utils.zenodo import create_zenodo_dataset, prepare_zenodo_dataset


def create_parser(add_help=True):
    parser = cli.Parser(add_help=add_help)

    parser.add_argument('--codemeta-location', dest='CODEMETA_LOCATION',
                        help='Location of the main codemeta.json JSON file')
    parser.add_argument('--zenodo-url', dest='ZENODO_URL', required=True,
                        help='URL of the Zenodo service. Test environment available at https://sandbox.zenodo.org')
    parser.add_argument('--zenodo-token', dest='ZENODO_TOKEN', required=True,
                        help='Zenodo personal token.')
    parser.add_argument('--zenodo-version-update', dest='ZENODO_VERSION_UPDATE', default=None,
                        help='Enable Zenodo version update. Can be "codemeta" or a Zenodo identifier. '
                        'If omitted, a new Zenodo dataset is created without versioning. '
                        'If set to "codemeta", a Zenodo identifier is searched in the CodeMeta file, and a new '
                        'version is created from it (if found). '
                        'Any other value is considered as a Zenodo identifier: a new version will be created from it. '
                        )
    parser.add_argument('--keep-previous-doi', action='store_true', dest='KEEP_PREVIOUS_DOI',
                        help='When creating a new version, keep the previous DOI in the Codemeta file, '
                        'in the field "isBasedOn". '
                        'By default, the previous DOI is removed.')
    parser.add_argument('--dry', action='store_true', dest='DRY',
                        help='Perform a dry run, do not upload anything.')
    parser.add_argument('--log-level', dest='LOG_LEVEL', default='WARN',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='LOG_FILE',
                        help='Path to the log file')
    return parser


def main(args):
    if args.CODEMETA_LOCATION:
        if is_remote(args.CODEMETA_LOCATION):
            raise ParserError(f'Location {args.CODEMETA_LOCATION} is not a local file.')

        codemeta = CodemetaMetadata()
        codemeta.fetch(args.CODEMETA_LOCATION)
        name = '{name} ({version}, in preparation)'.format(**codemeta.data)
    else:
        name = 'in preparation'

    zenodo_metadata = ZenodoMetadata({'name': name})
    zenodo_dict = zenodo_metadata.as_dict()

    # Management of Zenodo versioning
    old_zenodo_id = None
    if args.CODEMETA_LOCATION and args.ZENODO_VERSION_UPDATE == 'codemeta':
        # Check if CodeMeta file contains a Zenodo identifier
        identifiers = codemeta.data.get('identifier', [])
        if not isinstance(identifiers, list):
            identifiers = [identifiers]
        for identifier in identifiers:
            if isinstance(identifier, dict) and identifier.get('propertyID') == 'Zenodo':
                old_zenodo_id = identifier.get('value', None)
    elif args.ZENODO_VERSION_UPDATE is not None:
        # Use the provided Zenodo identifier to create a new version
        old_zenodo_id = args.ZENODO_VERSION_UPDATE

    if not args.DRY:
        # create Zenodo dataset
        dataset_id = create_zenodo_dataset(
            args.ZENODO_URL, args.ZENODO_TOKEN, zenodo_dict, previous_version=old_zenodo_id)
        dataset = prepare_zenodo_dataset(args.ZENODO_URL, dataset_id, args.ZENODO_TOKEN)

        doi = dataset.get('metadata', {}).get('doi', {})

        # Update Codemeta file with DOI and Zenodo ID
        if args.CODEMETA_LOCATION:
            codemeta.update_identifier(new_doi=doi, new_archive_id=dataset_id, archive_type='Zenodo',
                                       keep_previous_doi=args.KEEP_PREVIOUS_DOI)

            Path(args.CODEMETA_LOCATION).expanduser().write_text(codemeta.to_json())
        else:
            print(dataset)
    else:
        print(zenodo_dict)


def main_deprecated():
    cli.main_deprecated(__name__)


if __name__ == "__main__":
    main_deprecated()



"""FACILE-RS command-line tool, used to call the different scripts of the FACILE-RS project.

Description
-----------

This script is the entry point of the FACILE-RS command-line tool.
It is used to call the different scripts of the FACILE-RS project.
Use subcommands to select a platform (Zenodo, RADAR, ...) or metadata type (CFF, DataCite,...).
Use subsubcommands to select a FACILE-RS functionality.

Usage
-----

.. argparse::
    :module: facile_rs.utils.cli
    :func: create_parser
    :prog: facile-rs

Examples
--------

To generate CFF metadata from a CodeMeta metadata file:

    $ facile-rs cff create --codemeta-location codemeta.json
"""

import argparse
import logging
import os
import sys
from pathlib import Path
from warnings import warn

from dotenv import load_dotenv
from rich.logging import RichHandler

from facile_rs import (
    create_bag,
    create_bagpack,
    create_cff,
    create_datacite,
    create_radar,
    create_release,
    create_zenodo,
    prepare_radar,
    prepare_release,
    prepare_zenodo,
    run_bibtex_pipeline,
    run_docstring_pipeline,
    run_experiments_pipeline,
    run_markdown_pipeline,
)
from facile_rs.utils.exceptions import ParserError


class Parser(argparse.ArgumentParser):
    """
    Custom ArgumentParser class, which adds the functionality to add default values from the environment.
    """

    def add_argument(self, *args, **kwargs):
        env = kwargs.pop('env', True)
        argument = super().add_argument(*args, **kwargs)

        if not isinstance(argument, argparse._HelpAction):
            # update the default value for the argument with a possible environment variable
            if env:
                default = os.getenv(argument.dest)
                if default is not None:
                    # if the argument is a list, split the default from the environment
                    if kwargs.get('nargs') == '*' or kwargs.get('action') == 'append':
                        default = default.split()

                    argument.default = default

            # remove the required argument if a default is set
            if argument.required and argument.default:
                argument.required = False

        return argument

    def get_subparser(self, subcommand):
        # returns the subparser for a given subcommand
        for action in self._actions:
            if isinstance(action, argparse._SubParsersAction):
                subparser = action.choices[subcommand]
                return subparser


# create the top-level parser
def create_parser():
    """
    Create parsers for the facile-rs command line interface.
    The main parser has one subparser per platform (Zenodo, RADAR, ...) or metadata type (CFF, DataCite,...).
    Each of this subparser has a subparser per FACILE-RS script.

    :return: The parser object.
    """

    # Main parser
    parser = Parser(prog='facile-rs',
        description="FACILE-RS command-line tool, to perform metadata conversion and software publication" \
            " based on CodeMeta metadata.",
        epilog="Get help on a subcommand by running 'facile-rs <subcommand> -h'.")

    # Parsers for the subcommands
    subparsers = parser.add_subparsers(help='Select the target platform or metadata type.',
                                       dest='subcommand')

    # Parser for the 'release' subcommand
    parser_release = subparsers.add_parser('release', help='Perform operations on CodeMeta metadata')
    release_subparsers = parser_release.add_subparsers()
    parser_release_prepare = release_subparsers.add_parser('prepare',
                                                           help='Update CodeMeta file with the given version and date',
                                                           parents=[prepare_release.create_parser(add_help=False)],
                                                           add_help=True)
    parser_release_prepare.set_defaults(module=prepare_release)

    # Parser for the 'gitlab' subcommand
    parser_gitlab = subparsers.add_parser('gitlab', help='Perform operations for GitLab releases')
    gitlab_subparsers = parser_gitlab.add_subparsers()
    parser_gitlab_publish = gitlab_subparsers.add_parser('publish',
                                                        parents=[create_release.create_parser(add_help=False)],
                                                        help="Create a release on GitLab",
                                                        add_help=True)
    parser_gitlab_publish.set_defaults(module=create_release)

    # Parser for the 'radar' subcommand
    parser_radar = subparsers.add_parser('radar', help='Perform operations for RADAR releases')
    radar_subparsers = parser_radar.add_subparsers()
    parser_radar_prepare = radar_subparsers.add_parser('prepare',
                                                       help='Prepare a release on RADAR',
                                                       parents=[prepare_radar.create_parser(add_help=False)],
                                                       add_help=True)
    parser_radar_prepare.set_defaults(module=prepare_radar)
    parser_radar_upload = radar_subparsers.add_parser('upload',
                                                      help='Create a release on RADAR',
                                                      parents=[create_radar.create_parser(add_help=False)],
                                                      add_help=True)
    parser_radar_upload.set_defaults(module=create_radar)

    # Parser for the 'zenodo' subcommand
    parser_zenodo = subparsers.add_parser('zenodo', help='Perform operations for Zenodo releases')
    zenodo_subparsers = parser_zenodo.add_subparsers()
    parser_zenodo_prepare = zenodo_subparsers.add_parser('prepare',
                                                        help='Prepare a release on Zenodo',
                                                        parents=[prepare_zenodo.create_parser(add_help=False)],
                                                        add_help=True)
    parser_zenodo_prepare.set_defaults(module=prepare_zenodo)
    parser_zenodo_upload = zenodo_subparsers.add_parser('upload',
                                                        help='Create a release on Zenodo',
                                                        parents=[create_zenodo.create_parser(add_help=False)],
                                                        add_help=True)
    parser_zenodo_upload.set_defaults(module=create_zenodo)

    # Parser for the 'cff' subcommand
    parser_cff = subparsers.add_parser('cff', help='Generate and manage CFF metadata')
    cff_subparsers = parser_cff.add_subparsers()
    parser_cff_create = cff_subparsers.add_parser('create',
                                                  help='Create a CFF metadata file',
                                                  parents=[create_cff.create_parser(add_help=False)],
                                                  add_help=True)
    parser_cff_create.set_defaults(module=create_cff)

    # Parser for the 'datacite' subcommand
    parser_datacite = subparsers.add_parser('datacite', help='Generate and manage DataCite metadata')
    datacite_subparsers = parser_datacite.add_subparsers()
    parser_datacite_create = datacite_subparsers.add_parser('create',
                                                            help='Create a DataCite metadata file',
                                                            parents=[create_datacite.create_parser(add_help=False)],
                                                            add_help=True)
    parser_datacite_create.set_defaults(module=create_datacite)

    # Parser for the 'bag' subcommand
    parser_bag = subparsers.add_parser(
        'bag',
        help='Generate and manage BagIt bags',
        description='Generate and manage BagIt bags.'
    )
    bag_subparsers = parser_bag.add_subparsers()
    parser_bag_create = bag_subparsers.add_parser(
        'create',
        help='Create a BagIt bag',
        description='Create a BagIt bag.',
        parents=[create_bag.create_parser(add_help=False)],
        add_help=True
    )
    parser_bag_create.set_defaults(module=create_bag)

    # Parser for the 'bagpack' subcommand
    parser_bagpack = subparsers.add_parser('bagpack',
                                           help='Generate and manage BagPack bags (BagIt with DataCite metadata)')
    bagpack_subparsers = parser_bagpack.add_subparsers()
    parser_bagpack_create = bagpack_subparsers.add_parser('create',
                                                          help='Create a BagIt bag with DataCite metadata',
                                                          parents=[create_bagpack.create_parser(add_help=False)],
                                                          add_help=True)
    parser_bagpack_create.set_defaults(module=create_bagpack)

    # Parser for the 'grav' subcommand
    parser_grav = subparsers.add_parser('grav', help='Perform operations for Grav CMS')
    grav_subparsers = parser_grav.add_subparsers()
    parser_grav_bibtex = grav_subparsers.add_parser('bibtex',
                                                    help='Run the BibTex conversion pipeline',
                                                    parents=[run_bibtex_pipeline.create_parser(add_help=False)],
                                                    add_help=True)
    parser_grav_bibtex.set_defaults(module=run_bibtex_pipeline)

    parser_grav_docstring = grav_subparsers.add_parser('docstring',
                                                         help='Run the docstring conversion pipeline',
                                                         parents=[run_docstring_pipeline.create_parser(add_help=False)],
                                                         add_help=True)
    parser_grav_docstring.set_defaults(module=run_docstring_pipeline)

    parser_grav_experiments = grav_subparsers.add_parser('experiments',
                                                     help='Run the experiments conversion pipeline',
                                                     parents=[run_experiments_pipeline.create_parser(add_help=False)],
                                                     add_help=True)
    parser_grav_experiments.set_defaults(module=run_experiments_pipeline)

    parser_grav_markdown = grav_subparsers.add_parser('markdown',
                                                        help='Run the Markdown conversion pipeline',
                                                        parents=[run_markdown_pipeline.create_parser(add_help=False)],
                                                        add_help=True)
    parser_grav_markdown.set_defaults(module=run_markdown_pipeline)
    return parser


def setup_env():
    load_dotenv(Path().cwd() / '.env')


def setup_logs(log_level, log_file):
    log_level = log_level.upper()
    if log_file is None:
        logging.basicConfig(level=log_level, format='%(message)s', handlers=[RichHandler()])
    else:
        logging.basicConfig(level=log_level, format='[%(asctime)s] %(levelname)s %(name)s: %(message)s',
                            filename=Path(log_file).expanduser().as_posix())

def main():
    # load the environment
    setup_env()

    # Create the command-line parser
    parser = create_parser()
    args = parser.parse_args()

    # Print help if subcommand is missing
    subcommand = args.subcommand
    if subcommand is None:
        parser.print_help()
    else:
        try:
            module = args.module
        except AttributeError:
            subparser = parser.get_subparser(subcommand)
            subparser.print_help()
        else:
            # setup logs
            setup_logs(args.LOG_LEVEL, args.LOG_FILE)

            # Call the "main" function of the module
            try:
                module.main(args)
            except ParserError as e:
                parser.error(e)


def main_deprecated(module_name):
    """
    Display a deprecation warning when a script is called from the command line directly,
    without using the 'facile-rs' entry point.

    :param func: The main function to call in the script.
    """

    # create the deprecation warning
    module = sys.modules[module_name]
    script_name = module.__name__.split('.')[-1]
    warn(f"Calling {script_name} directly is deprecated. Use the entry point 'facile-rs' instead.", stacklevel=2)

    # load the environment
    setup_env()

    # parse the arguments
    parser = module.create_parser()
    args = parser.parse_args()

    # setup logs
    setup_logs(args.LOG_LEVEL, args.LOG_FILE)

    # call the main function
    module.main(args)



import pytest


@pytest.fixture(autouse=True)
def unset_facilers_env(monkeypatch):
    """
    Ensure not to use variables defined in external environment when running tests.
    """
    env_vars = [
        'ASSETS',
        'BAG_INFO_LOCATION',
        'BAG_PATH',
        'CFF_PATH',
        'CODEMETA_LOCATION',
        'CONTRIBUTORS_LOCATIONS',
        'CREATORS_LOCATIONS',
        'DATE',
        'DATACITE_PATH',
        'GRAV_PATH',
        'LOG_FILE',
        'LOG_LEVEL',
        'NOTIFICATION_EMAIL',
        'PIPELINE',
        'PIPELINE_CSL',
        'PIPELINE_FOOTER',
        'PIPELINE_HEADER',
        'PIPELINE_IMAGES',
        'PIPELINE_REFS',
        'PIPELINE_SOURCE',
        'PRIVATE_TOKEN',
        'RADAR_BACKLINK',
        'RADAR_CLIENT_ID',
        'RADAR_CLIENT_SECRET',
        'RADAR_EMAIL',
        'RADAR_PASSWORD',
        'RADAR_PATH',
        'RADAR_REDIRECT_URL',
        'RADAR_URL',
        'RADAR_USERNAME',
        'RADAR_WORKSPACE_ID',
        'RELEASE_API_URL',
        'RELEASE_DESCRIPTION',
        'RELEASE_TAG',
        'SMTP_SERVER',
        'VERSION',
        'ZENODO_PATH',
        'ZENODO_TOKEN',
        'ZENODO_URL',
    ]
    for env_var in env_vars:
        monkeypatch.delenv(env_var, raising=False)
    yield



#!/usr/bin/env python3

"""Compile and copy the content of bibtex files to a Grav CMS repository.

Description
-----------

This script compiles and copies the content of bibtex files in a similar way as run_markdown_pipeline.
A CSL can be provided.
Please refer to https://git.opencarp.org/openCARP/publications for an example setup.

Usage
-----

.. argparse::
    :module: facile_rs.run_bibtex_pipeline
    :func: create_parser
    :prog: run_bibtex_pipeline.py

"""
import logging
from pathlib import Path

import frontmatter
import pypandoc

from .utils import cli
from .utils.grav import collect_pages

logger = logging.getLogger(__file__)

TEMPLATE = '''
---
nocite: '@*'
---
'''


def create_parser(add_help=True):
    parser = cli.Parser(add_help=add_help)

    parser.add_argument('--grav-path', dest='GRAV_PATH', required=True,
                        help='Path to the grav repository directory.')
    parser.add_argument('--pipeline', dest='PIPELINE', required=True,
                        help='Name of the pipeline as specified in the GRAV metadata.')
    parser.add_argument('--pipeline-source', dest='PIPELINE_SOURCE', required=True,
                        help='Path to the source directory for the pipeline.')
    parser.add_argument('--pipeline-csl', dest='PIPELINE_CSL',
                        help='Path to the source directory for the pipeline.')
    parser.add_argument('--log-level', dest='LOG_LEVEL', default='WARN',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='LOG_FILE',
                        help='Path to the log file')
    return parser


def main(args):
    # loop over the found pages and write the content into the files
    for page_path, page, source in collect_pages(args.GRAV_PATH, args.PIPELINE):
        source_path = Path(args.PIPELINE_SOURCE).expanduser() / source
        logger.debug('page_path = %s, source_path = %s', page_path, source_path)

        extra_args = [f'--bibliography={source_path}', '--citeproc', '--wrap=preserve']
        if args.PIPELINE_CSL:
            extra_args.append(f'--csl={args.PIPELINE_CSL}')

        page.content = pypandoc.convert_text(TEMPLATE, to='html', format='md',
                                             extra_args=extra_args)

        logger.info('writing publications to %s', page_path)
        open(page_path, 'w').write(frontmatter.dumps(page))


def main_deprecated():
    cli.main_deprecated(__name__)


if __name__ == "__main__":
    main_deprecated()



#!/usr/bin/env python3

"""Create a BagIt bag from a list of assets.

Description
-----------

This script creates a BagIt bag using the bagit-python package.
The assets to be included in the bag are given as positional arguments.

Usage
-----

.. argparse::
    :module: facile_rs.create_bag
    :func: create_parser
    :prog: create_bag.py

"""
import bagit

from .utils import cli, setup_assets_path
from .utils.exceptions import ParserError
from .utils.http import fetch_dict, fetch_files


def create_parser(add_help=True):
    parser = cli.Parser(add_help=add_help)

    parser.add_argument('ASSETS', nargs='*', default=[],
                        help='Assets to be added to the bag.')
    parser.add_argument('--bag-path', dest='BAG_PATH', required=True,
                        help='Path to the Bag directory')
    parser.add_argument('--bag-info-locations', '--bag-info-location', dest='BAG_INFO_LOCATIONS',
                        action='append', default=[],
                        help='Locations of the bag-info YAML/JSON files')
    parser.add_argument('--assets-token', dest='ASSETS_TOKEN',
                        help='Private token, to be used when fetching assets')
    parser.add_argument('--assets-token-name', dest='ASSETS_TOKEN_NAME', default='PRIVATE-TOKEN',
                        help='Name of the header field for the token [default: "PRIVATE-TOKEN"]')
    parser.add_argument('--overwrite', dest='OVERWRITE', action='store_true', env=False,
                        help='Overwrite existing Bag directory')
    parser.add_argument('--log-level', dest='LOG_LEVEL', default='WARN',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='LOG_FILE',
                        help='Path to the log file')

    return parser


def main(args):
    # setup the bag
    try:
        bag_path = setup_assets_path(args.BAG_PATH, remove_existing=args.OVERWRITE)
    except FileExistsError as e:
        raise ParserError(f'{args.BAG_PATH} already exists. Please remove it or use --overwrite to replace it.') from e

    # collect assets
    fetch_files(args.ASSETS, bag_path, headers={
        args.ASSETS_TOKEN_NAME: args.ASSETS_TOKEN
    } if args.ASSETS_TOKEN else {})

    # fetch bag-info
    bag_info = {}
    for location in args.BAG_INFO_LOCATIONS:
        bag_info.update(fetch_dict(location))

    # create bag using bagit
    bag = bagit.make_bag(bag_path, bag_info)
    bag.save()


def main_deprecated():
    cli.main_deprecated(__name__)


if __name__ == "__main__":
    main_deprecated()



import ast
import sys

from facile_rs.utils.cli import main

RELEASE_TAG = 'v0.1.0'
RELEASE_DESCRIPTION = 'Release for version 0.1.0'
RELEASE_API_URL = 'https://gitlab.example.com/api/v4/projects/123/releases'
PRIVATE_TOKEN = '1234567890'
ASSETS = ['https://example.com/assets/asset1', 'https://example.com/assets/asset2']


def test_create_release_dry_cli(monkeypatch, capsys):
    """"
    Check release request content in dry mode using CLI options.
    """
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'gitlab',
                            'publish',
                            '--release-tag', RELEASE_TAG,
                            '--release-description', RELEASE_DESCRIPTION,
                            '--release-api-url', RELEASE_API_URL,
                            '--private-token', PRIVATE_TOKEN,
                            '--dry',
                            *ASSETS
                        ])
    main()
    captured = ast.literal_eval(capsys.readouterr().out)
    assert captured['name'] == RELEASE_TAG
    assert captured['tag_name'] == RELEASE_TAG
    assert captured['description'] == RELEASE_DESCRIPTION.strip()
    assert len(captured['assets']['links']) == 2
    assert captured['assets']['links'][0]['url'] == 'https://example.com/assets/asset1'
    assert captured['assets']['links'][1]['name'] == 'asset2'


def test_create_release_dry_env(monkeypatch, capsys):
    """
    Check release request content in dry mode using environment variables.
    """
    monkeypatch.setenv('RELEASE_TAG', RELEASE_TAG)
    monkeypatch.setenv('RELEASE_DESCRIPTION', RELEASE_DESCRIPTION)
    monkeypatch.setenv('RELEASE_API_URL', RELEASE_API_URL)
    monkeypatch.setenv('PRIVATE_TOKEN', PRIVATE_TOKEN)
    monkeypatch.setenv('ASSETS', ' '.join(ASSETS))
    monkeypatch.setattr('sys.argv',
                       [
                           sys.argv[0],
                           'gitlab',
                           'publish',
                           '--dry'
                       ])
    main()
    captured = ast.literal_eval(capsys.readouterr().out)
    assert captured['name'] == RELEASE_TAG
    assert captured['tag_name'] == RELEASE_TAG
    assert captured['description'] == RELEASE_DESCRIPTION.strip()
    assert len(captured['assets']['links']) == 2
    assert captured['assets']['links'][1]['url'] == 'https://example.com/assets/asset2'
    assert captured['assets']['links'][0]['name'] == 'asset1'



from os import path

from facile_rs.utils.metadata import ZenodoMetadata

# Get current script location
SCRIPT_DIR = path.dirname(path.realpath(__file__))


def test_get_license_id_from_spdx():
    metadata = ZenodoMetadata()
    assert metadata.get_license_id_from_spdx("LGPL-3.0") == "lgpl-3.0-only"
    assert metadata.get_license_id_from_spdx("MIT") == "mit"
    assert metadata.get_license_id_from_spdx("Apache-2.0") == "apache-2.0"
    assert metadata.get_license_id_from_spdx("not-a-license") is None


def test_to_person_or_org():
    metadata = ZenodoMetadata()
    codemeta_person = {
        "@type": "Person",
        "@id": "https://orcid.org/0000-0000-0000-0000",
        "givenName": "John",
        "familyName": "Smith",
        "affiliation": [
            {
                "@type": "Organization",
                "@id": "https://ror.org/04t3en479",
                "name": "Karlsruhe Institute of Technology (KIT)"
            },
            {
                "@type": "Organization",
                "@id": "https://ror.org/0245cg223",
                "name": "University of Freiburg"
            }
        ]
    }
    zenodo_person = {
        "person_or_org": {
            "type": "personal",
            "given_name": "John",
            "family_name": "Smith",
            "identifiers": [
                {
                    "scheme": "orcid",
                    "identifier": "0000-0000-0000-0000"
                }
            ]
        },
        "affiliations": [
            {
                "name": "Karlsruhe Institute of Technology (KIT)",
                "id": "04t3en479"
            },
            {
                "name": "University of Freiburg",
                "id": "0245cg223"
            }
        ]
    }
    codemeta_organization = {
        "@type": "Organization",
        "@id": "https://ror.org/02n0bts35",
        "name": "Medical University of Graz"
        }
    zenodo_organization = {
        "person_or_org": {
            "type": "organizational",
            "name": "Medical University of Graz",
            "identifiers": [
                {
                    "scheme": "ror",
                    "identifier": "02n0bts35"
                }
        ]
        }
    }
    codemeta_person_single_affiliation = {
        "id": "https://orcid.org/0000-0002-6584-0233",
        "type": "Person",
        "affiliation": {
            "type": "Organization",
            "name": "Karlsruhe Institute of Technology"
        },
        "email": "marie.houillon@kit.edu",
        "familyName": "Houillon",
        "givenName": "Marie"
    }
    zenodo_person_single_affiliation = {
        "person_or_org": {
            "type": "personal",
            "given_name": "Marie",
            "family_name": "Houillon",
            "identifiers": [
                {
                    "scheme": "orcid",
                    "identifier": "0000-0002-6584-0233"
                }
            ]
        },
        "affiliations": [
            {
                "name": "Karlsruhe Institute of Technology"
            }
        ]
    }

    zenodo_contributor = zenodo_person.copy()
    zenodo_contributor["role"] = {"id": "datacurator"}
    zenodo_unknown_contributortype = zenodo_person.copy()
    zenodo_unknown_contributortype["role"] = {"id": "other"}

    assert metadata.to_person_or_org(codemeta_person) == zenodo_person
    assert metadata.to_person_or_org(codemeta_person_single_affiliation) == zenodo_person_single_affiliation
    assert metadata.to_person_or_org(codemeta_organization) == zenodo_organization
    assert metadata.to_person_or_org(codemeta_person, "datacurator") == zenodo_contributor
    assert metadata.to_person_or_org(codemeta_person, "InvalidType") == zenodo_unknown_contributortype


def test_to_funding():
    metadata = ZenodoMetadata()
    codemeta_invalid_funding = {
      "@type": "Grant",
      "name": "Rolling out the openCARP simulator as a sustainable e-research infrastructure: CARPe-diem",
      "identifier": 507828355,
      "url": "https://gepris.dfg.de/gepris/projekt/507828355?language=en",
      "funder": {
        "@type": "Organization",
        "@id": "https://ror.org/018mejw64",
        "name": "Deutsche Forschungsgemeinschaft"
      }
    }
    codemeta_valid_funding = {
      "@type": "Grant",
      "name": "Numerical modeling of cardiac electrophysiology at the cellular scale",
      "identifier": 955495,
      "url": "https://eurohpc-ju.europa.eu/actions#ecl-inpage-259",
      "funder": {
        "@type": "Organization",
        "@id": "https://ror.org/00k4n6c32",
        "name": "European High-Performance Computing Joint Undertaking EuroHPC (JU)"
      }
    }
    zenodo_invalid_grant = {
        "funder": {
          "id": "018mejw64"
        },
        "award": {
          "title": {
            "en": "Rolling out the openCARP simulator as a sustainable e-research infrastructure: CARPe-diem"
          },
          "number": 507828355,
          "identifiers": [
            {
              "scheme": "url",
              "identifier": "https://gepris.dfg.de/gepris/projekt/507828355?language=en"
            }
          ]
        }
      }
    zenodo_valid_grant = {
        "award": {
          "id": "00k4n6c32::955495"
        },
        "funder": {
          "id": "00k4n6c32"
        }
      }
    assert metadata.to_funding(codemeta_valid_funding) == zenodo_valid_grant
    assert metadata.to_funding(codemeta_invalid_funding) == zenodo_invalid_grant
    metadata.data = {"funder": "The grant funder", "funding": "The grant name"}
    zenodo_funding = {
        "award": {
            "title": {"en": "The grant name"}
        },
        "funder": {
            "name": "The grant funder"
        }
    }
    assert metadata.to_funding("The grant name") == zenodo_funding

def test_to_rights():
    metadata = ZenodoMetadata()
    codemeta_license = "https://spdx.org/licenses/MIT"
    zenodo_right = {"id": "mit"}
    assert metadata.to_rights(codemeta_license) == zenodo_right
    codemeta_license = "MIT"
    assert metadata.to_rights(codemeta_license) == zenodo_right
    codemeta_license = "not-a-license"
    assert metadata.to_rights(codemeta_license) == {}
    codemeta_license = {
        "@type": "CreativeWork",
        "name": "ACADEMIC PUBLIC LICENSE (openCARP, v1.1)",
        "url": "https://openCARP.org/download/license"
    }
    zenodo_right = {
        "title": {
            "en": "ACADEMIC PUBLIC LICENSE (openCARP, v1.1)"
        },
        "link": "https://openCARP.org/download/license"
    }
    assert metadata.to_rights(codemeta_license) == zenodo_right
    codemeta_license = {
        "@type": "CreativeWork",
        "name": "Apache License, Version 2.0",
        "url": "https://spdx.org/licenses/Apache-2.0"
    }
    zenodo_right = {
        "id": "apache-2.0"
    }
    assert metadata.to_rights(codemeta_license) == zenodo_right

def test_validate_funding_identifier():
    metadata = ZenodoMetadata()
    assert metadata.validate_funding_identifier("955495")['id'] == "00k4n6c32::955495"
    assert metadata.validate_funding_identifier(955495)['id'] == "00k4n6c32::955495"
    assert metadata.validate_funding_identifier("nonvalid1234") == {}

def test_validate_funder_identifier():
    metadata = ZenodoMetadata()
    assert metadata.validate_funder_identifier("https://ror.org/00k4n6c32") == "00k4n6c32"
    assert metadata.validate_funder_identifier("00k4n6c32") == "00k4n6c32"
    assert metadata.validate_funder_identifier("nonvalid1234") is None

def test_to_funder():
    metadata = ZenodoMetadata()
    codemeta_funder = {
        "@type": "Organization",
        "@id": "https://ror.org/00k4n6c32",
        "name": "European High-Performance Computing Joint Undertaking EuroHPC (JU)"
    }
    zenodo_funder = {
        "id": "00k4n6c32"
    }
    assert metadata.to_funder(codemeta_funder) == zenodo_funder
    codemeta_funder = {
        "type": "Organization",
        "name": "The funding organization",
        "url": "https://thefundingorganization.org"
    }
    zenodo_funder = {
        "name": "The funding organization"
    }
    assert metadata.to_funder(codemeta_funder) == zenodo_funder

def test_to_related_identifier():
    metadata = ZenodoMetadata()
    identifier = "https://git.opencarp.org/openCARP/openCARP"
    relation = "issupplementto"
    zenodo_related_identifier = {
        "relation_type": {"id": relation},
        "identifier": identifier
    }
    assert metadata.to_related_identifier(identifier, relation) == zenodo_related_identifier
    relation = "notincontrolledvocabulary"
    assert metadata.to_related_identifier(identifier, relation) is None

def test_to_subjects():
    metadata = ZenodoMetadata()
    codemeta_keywords = ["cardiac electrophysiology"]
    zenodo_subjects = [{"subject": "cardiac electrophysiology"}]
    assert metadata.to_subjects(codemeta_keywords) == zenodo_subjects
    codemeta_keywords = [{"@type": "DefinedTerm",
      "name": "Mathematical models",
      "url": "http://id.loc.gov/authorities/subjects/sh85082124",
      "inDefinedTermSet": "http://id.loc.gov/authorities/subjects"
    }]
    zenodo_subjects = [{"subject": "http://id.loc.gov/authorities/subjects/sh85082124"}]
    assert metadata.to_subjects(codemeta_keywords) == zenodo_subjects

def test_add_to_array_field():
    metadata = ZenodoMetadata()
    zenodo_dict = {'metadata': {}}
    field = 'contributors'
    value1 = {'name': 'John Smith'}
    metadata.add_to_array_field(zenodo_dict, field, value1)
    assert zenodo_dict['metadata'][field] == [value1]
    value2 = {'name': 'Jane Doe'}
    metadata.add_to_array_field(zenodo_dict, field, value2)
    assert isinstance(zenodo_dict['metadata'][field], list)
    assert len(zenodo_dict['metadata'][field]) == 2
    assert value1 in zenodo_dict['metadata'][field]
    assert value2 in zenodo_dict['metadata'][field]
